import React, {
  memo,
  forwardRef,
  Ref,
  useCallback,
  useImperativeHandle,
  useMemo,
} from 'react';
import {Keyboard, StyleSheet, View} from 'react-native';
import Animated, {
  useAnimatedGestureHandler,
  useSharedValue,
  useAnimatedStyle,
  runOnJS,
} from 'react-native-reanimated';
import {PanGestureHandler} from 'react-native-gesture-handler';
import RotationSnapPoint from '../rotationSnapPoint/RotationSnapPoint';
import ResizerSnapPoint from '../resizerSnapPoint/ResizerSnapPoint';
import CustomTextInput from '../customTextInput/CustomTextInput';
import {RNDTInternalProvider} from '../../context/internal';
import {RNDTExternalProvider} from '../../context/external';
import {
  defBoxHeight,
  defBoxWidth,
  defRotationAngle,
  defX,
  defY,
  radian,
  SIDES,
  textInputLayoutDefaults,
} from '../../constants';
import CornerComponent from '../cornerComponent/CornerComponent';
import {CornerComponentProps} from '../cornerComponent/types.d';
import {DragTextPropTypes, DragTextRef} from './types';

const DragText = (
  {
    rotationComponent,
    resizerSnapPoints,
    cornerComponents,
    externalTextStyles,
    externalBorderStyles,
    defaultTextValue,
    placeholder,
    visible = true,
    onItemActive,
  }: DragTextPropTypes,
  ref: Ref<DragTextRef>,
) => {
  const x = useSharedValue<number>(defX);
  const y = useSharedValue<number>(defY);
  const boxWidth = useSharedValue<number>(defBoxWidth);
  const boxHeight = useSharedValue<number>(defBoxHeight);
  const rotationAngle = useSharedValue<number>(defRotationAngle);
  const isResize = useSharedValue<boolean>(false);
  const borderStatus = useSharedValue<boolean>(true);
  const textInputLayout = textInputLayoutDefaults;

  const dragHandler = useAnimatedGestureHandler({
    onStart: (_ev: any, ctx: any) => {
      runOnJS(Keyboard.dismiss)();
      ctx.startX = x.value;
      ctx.startY = y.value;
    },
    onActive: (_ev, ctx) => {
      if (borderStatus.value) {
        y.value = ctx.startY + _ev.translationY;
        x.value = ctx.startX + _ev.translationX;
      }
    },
    onEnd: () => {
      borderStatus.value = false;
    },
  });

  useImperativeHandle(ref, () => ({
    setBorderStatus: param => {
      borderStatus.value = param;
    },
  }));

  const externalContextVariables = useMemo(
    () => ({
      resizerSnapPoints,
      cornerComponents,
      externalTextStyles,
      externalBorderStyles,
      rotationComponent,
      defaultTextValue,
      placeholder,
      onItemActive,
    }),
    [
      resizerSnapPoints,
      cornerComponents,
      externalTextStyles,
      externalBorderStyles,
      rotationComponent,
      defaultTextValue,
      placeholder,
      onItemActive,
    ],
  );

  const internalContextVariables = useMemo(
    () => ({
      x,
      y,
      boxWidth,
      boxHeight,
      rotationAngle,
      isResize,
      textInputLayout,
      borderStatus,
    }),
    [x, y, boxWidth, boxHeight, isResize, textInputLayout, borderStatus],
  );

  const animatedDragStyles = useAnimatedStyle(
    () => ({
      transform: [{translateX: x.value}, {translateY: y.value}],
      width: boxWidth.value,
      height: boxHeight.value,
      display: visible ? 'flex' : 'none',
    }),
    [x, y, boxWidth, visible, boxHeight, borderStatus],
  );

  const animatedRotationStyles = useAnimatedStyle(
    () => ({
      transform: [{rotateZ: rotationAngle.value + radian}],
      borderWidth: borderStatus.value ? 1 : 0,
    }),
    [rotationAngle, borderStatus],
  );

  const _cornerComponent = useCallback(
    ({side, customCornerComponent}: CornerComponentProps, i: number) => (
      <CornerComponent
        side={side}
        customCornerComponent={customCornerComponent}
        key={i}
      />
    ),
    [cornerComponents, borderStatus],
  );

  const _resizerSnapPoint = useCallback(
    (sides: SIDES | string, i: number) => (
      <ResizerSnapPoint side={sides} key={i} />
    ),
    [resizerSnapPoints, borderStatus],
  );

  return (
    <RNDTExternalProvider value={externalContextVariables}>
      <RNDTInternalProvider value={internalContextVariables}>
        <PanGestureHandler onGestureEvent={dragHandler}>
          <Animated.View style={[animatedDragStyles, styles.dragContainer]}>
            <Animated.View
              style={[
                animatedRotationStyles,
                styles.rotationStyles,
                externalBorderStyles,
              ]}>
              {rotationComponent && (
                <RotationSnapPoint
                  side={rotationComponent.side}
                  customRotationComponent={
                    rotationComponent.customRotationComponent
                  }
                />
              )}
              {cornerComponents?.map(_cornerComponent)}
              {resizerSnapPoints?.map(_resizerSnapPoint)}
              <View style={styles.textInputContainer}>
                <CustomTextInput />
              </View>
            </Animated.View>
          </Animated.View>
        </PanGestureHandler>
      </RNDTInternalProvider>
    </RNDTExternalProvider>
  );
};

const styles = StyleSheet.create({
  dragContainer: {
    position: 'absolute',
  },
  textInputContainer: {
    borderColor: 'transparent',
    justifyContent: 'center',
    alignItems: 'center',
  },
  rotationStyles: {
    ...StyleSheet.absoluteFillObject,
    justifyContent: 'center',
    alignItems: 'center',
    borderWidth: 1,
    borderStyle: 'dashed',
    borderColor: '#777',
  },
});

export default memo(forwardRef(DragText));
