import React, {useCallback, useState} from 'react';
import Animated, {
  runOnJS,
  useAnimatedGestureHandler,
  useAnimatedStyle,
} from 'react-native-reanimated';
import {PanGestureHandler} from 'react-native-gesture-handler';
import styles from './styles';
import {useRNDTInternal} from '../../hooks';
import {ResizerSnapPointProps} from './types.d';
import {
  handleBottom,
  handleLeft,
  handleRight,
  handleTop,
} from '../../utils/calculate';
import {Keyboard, StyleProp, View, ViewStyle} from 'react-native';
import {SIDES, textInputLayoutOffset} from '../../constants';

const ResizerSnapPoint = ({side}: ResizerSnapPointProps) => {
  const {boxWidth, x, y, textInputLayout, boxHeight, isResize, borderStatus} =
    useRNDTInternal();

  const [widthenabled, setWidthEnabled] = useState(true);
  const [heightenabled, setHeightEnabled] = useState(true);

  const handleLimiters = useCallback(() => {
    if (side == SIDES.LEFT || side == SIDES.RIGHT) {
      if (boxWidth.value <= textInputLayout.width) {
        setWidthEnabled(false);
      } else {
        setWidthEnabled(true);
      }
    } else {
      if (boxHeight.value <= textInputLayout.height) {
        setHeightEnabled(false);
      } else {
        setHeightEnabled(true);
      }
    }
  }, [side, boxHeight, boxWidth]);

  const handleResizers = (params: any) => {
    'worklet';
    switch (side) {
      case SIDES.LEFT:
        [boxWidth.value, x.value] = handleLeft(params);
        break;
      case SIDES.BOTTOM:
        boxHeight.value = handleBottom(params);
        break;
      case SIDES.RIGHT:
        boxWidth.value = handleRight(params);
        break;
      case SIDES.TOP:
        [y.value, boxHeight.value] = handleTop(params);
        break;
      case undefined:
        break;
    }
    isResize.value = false;
    runOnJS(handleLimiters)();
  };

  const resizeHandler = useAnimatedGestureHandler(
    {
      onStart: (_ev: any, ctx: any) => {
        runOnJS(Keyboard.dismiss)();
        ctx.boxH = boxHeight.value + textInputLayoutOffset;
        ctx.boxW = boxWidth.value + textInputLayoutOffset;
        ctx.startX = x.value;
        ctx.startY = y.value;
      },
      onActive: (_ev, ctx) => {
        handleResizers({_ev, ctx});
      },
      onFinish: (_ev, ctx) => {
        runOnJS(setWidthEnabled)(true);
        runOnJS(setHeightEnabled)(true);
      },
    },
    [],
  );

  /**
   *  resizeFieldStyles: transparent snap point with bigger
   *  touchable field than visible white snap points
   */
  const resizeFieldStyles: {[key in SIDES]: StyleProp<ViewStyle>} = {
    left: styles.leftResizerField,
    right: styles.rightResizerField,
    bottom: styles.bottomResizerField,
    top: styles.topResizerField,
  };
  const resizeFieldStyle = resizeFieldStyles[side as SIDES];

  /** resizeStyles: visible white snap point */
  const resizeStyles: {[key in SIDES]: StyleProp<ViewStyle>} = {
    left: styles.leftResizer,
    right: styles.rightResizer,
    bottom: styles.bottomResizer,
    top: styles.topResizer,
  };
  const resizeStyle = resizeStyles[side as SIDES];

  const borderStatusStyle = useAnimatedStyle(() => ({
    display: borderStatus.value ? 'flex' : 'none',
  }));

  return (
    <PanGestureHandler
      enabled={widthenabled && heightenabled}
      onGestureEvent={resizeHandler}>
      <Animated.View style={[resizeFieldStyle, borderStatusStyle]}>
        <View style={resizeStyle}></View>
      </Animated.View>
    </PanGestureHandler>
  );
};

export default ResizerSnapPoint;
